package com.vcarecity.cvs;

import com.vcarecity.cvs.entity.CheckRecordEntity;
import com.vcarecity.cvs.factory.ResultHandlerFactory;
import com.vcarecity.cvs.properties.AppProperties;
import com.vcarecity.cvs.service.ResultHandlerService;
import com.vcarecity.cvs.service.SQLQueryService;
import com.vcarecity.cvs.service.impl.CSVResultHandlerServiceImpl;
import lombok.extern.slf4j.Slf4j;

import javax.inject.Inject;
import java.util.List;

/**
 * @author Kerry on 19/11/20
 */

@Slf4j
public class MySQLStarter {
    private final AppProperties properties;
    private final SQLQueryService queryService;
    private final ResultHandlerFactory resultHandlerFactory;

    @Inject
    public MySQLStarter(AppProperties properties,
                        SQLQueryService queryService,
                        ResultHandlerFactory resultHandlerFactory) {

        this.properties = properties;
        this.queryService = queryService;
        this.resultHandlerFactory = resultHandlerFactory;

    }

    static final int PAGE_COUNT = 10000;

    public void run() throws Exception {

        logger.info("start query database...");

        long startTime = System.currentTimeMillis();


        String sql = "select * from T_CHECK_RECORD limit ?,?";
        // sql = "SELECT * FROM websites  limit ?,?";

        String properties = "check_record.csv";

        int count = 0;
        int currentSize;


        ResultHandlerService resultHandler = resultHandlerFactory.createResultHandler(properties);

        do {
            List<CheckRecordEntity> result = queryService.queryByPage(sql, count, PAGE_COUNT, CheckRecordEntity.class);
            resultHandler.resultHandler(result, CheckRecordEntity.class);
            currentSize = result.size();
            count += currentSize;

        } while (currentSize == PAGE_COUNT);

        if (resultHandler instanceof CSVResultHandlerServiceImpl) {
            ((CSVResultHandlerServiceImpl) resultHandler).close();
        }

        logger.info("stop query database...{} /ms.", (System.currentTimeMillis() - startTime));
    }

}
