package com.vcarecity.cvs.core;

import javax.persistence.Column;
import javax.persistence.Transient;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author Kerry on 19/11/21
 */

public class ReflectionUtil {


    private static final Map<Class<?>, List<JavaMapColumn>> FIELD_CACHE = new ConcurrentHashMap<>();


    public static List<JavaMapColumn> getFieldColumnList(Class<?> cls) {

        List<JavaMapColumn> fields = FIELD_CACHE.get(cls);
        if (fields != null) {
            return fields;
        }
        synchronized (FIELD_CACHE) {
            Field[] declaredFields = cls.getDeclaredFields();
            fields = new ArrayList<>(declaredFields.length);

            for (Field declaredField : declaredFields) {
                Transient ta = declaredField.getAnnotation(Transient.class);
                if (ta != null) {
                    continue;
                }
                final Column column = declaredField.getAnnotation(Column.class);
                String fieldName = declaredField.getName();
                if (column != null) {
                    final String name = column.name();
                    if (name.length() > 0) {
                        fieldName = name;
                    }
                }
                fields.add(new JavaMapColumn(declaredField, fieldName));
            }
            FIELD_CACHE.put(cls, fields);
            return FIELD_CACHE.get(cls);
        }
    }

}
