package com.vcarecity.cvs.module;

import com.google.inject.AbstractModule;
import com.google.inject.Inject;
import com.google.inject.Provides;
import com.google.inject.Singleton;
import com.google.inject.assistedinject.FactoryModuleBuilder;
import com.vcarecity.cvs.factory.ResultHandlerFactory;
import com.vcarecity.cvs.properties.AppProperties;
import com.vcarecity.cvs.properties.DbProperties;
import com.vcarecity.cvs.service.ResultHandlerService;
import com.vcarecity.cvs.service.SQLQueryService;
import com.vcarecity.cvs.service.impl.CSVResultHandlerServiceImpl;
import com.vcarecity.cvs.service.impl.MySQLQueryServiceImpl;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;

/**
 * @author Kerry on 19/11/20
 */

public class MySQLModule extends AbstractModule {
    @Override
    protected void configure() {
        bind(SQLQueryService.class).to(MySQLQueryServiceImpl.class);

        install(new FactoryModuleBuilder()
                .implement(ResultHandlerService.class, CSVResultHandlerServiceImpl.class)
                .build(ResultHandlerFactory.class));

    }

    @Provides
    @Singleton
    @Inject
    public HikariDataSource dataSource(AppProperties properties) {
        // @see https://github.com/brettwooldridge/HikariCP/wiki/MySQL-Configuration

        final DbProperties database = properties.getDatabase();
        HikariConfig config = new HikariConfig();
        config.setJdbcUrl(database.getUrl());
        config.setUsername(database.getUser());
        config.setPassword(database.getPassword());
        config.addDataSourceProperty("cachePrepStmts", "true");
        config.addDataSourceProperty("prepStmtCacheSize", "250");
        config.addDataSourceProperty("prepStmtCacheSqlLimit", "2048");
        config.addDataSourceProperty("useServerPrepStmts", "true");
        config.addDataSourceProperty("useLocalSessionState", "true");
        config.addDataSourceProperty("rewriteBatchedStatements", "true");
        config.addDataSourceProperty("cacheResultSetMetadata", "true");
        config.addDataSourceProperty("cacheServerConfiguration", "true");
        config.addDataSourceProperty("elideSetAutoCommits", "true");
        config.addDataSourceProperty("maintainTimeStats", "false");

        final HikariDataSource hikariDataSource = new HikariDataSource(config);
        Runtime.getRuntime().addShutdownHook(new Thread(hikariDataSource::close));
        return hikariDataSource;
    }

}
