package com.vcarecity.cvs.service.impl;

import com.google.inject.Inject;
import com.vcarecity.cvs.core.AutoMapper;
import com.vcarecity.cvs.service.SQLQueryService;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Kerry on 19/11/21
 */


@Slf4j
public class MySQLQueryServiceImpl implements SQLQueryService {
    private final HikariDataSource dataSource;

    @Inject
    public MySQLQueryServiceImpl(HikariDataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public <T> List<T> queryByPage(String sql, int start, int pageCount, Class<T> clazz) throws SQLException {
        final Connection connection = dataSource.getConnection();
        final PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setInt(1, start);
        preparedStatement.setInt(2, pageCount);

        List<T> list = new ArrayList<>(pageCount);
        long startTime = System.currentTimeMillis();

        logger.debug("start query: {},{},{}.start on = {}", sql, start, pageCount, startTime);

        final ResultSet resultSet = preparedStatement.executeQuery();
        while (resultSet.next()) {
            // mapper
            final T t = AutoMapper.autoMapper(resultSet, clazz);
            if (t != null) {
                list.add(t);
            }
        }

        logger.debug("start query: {},{},{}.usageTime= {}", sql, start, pageCount, (System.currentTimeMillis() - startTime));
        connection.close();

        return list;
    }
}
